@@ -1,12 +1,12 @@
 # -*- coding: utf-8 -*-
 #
 #
-#  TheVirtualBrain-Scientific Package. This package holds all simulators, and 
+# TheVirtualBrain-Scientific Package. This package holds all simulators, and
 # analysers necessary to run brain-simulations. You can use it stand alone or
 # in conjunction with TheVirtualBrain-Framework Package. See content of the
 # documentation-folder for more details. See also http://www.thevirtualbrain.org
 #
-# (c) 2012-2020, Baycrest Centre for Geriatric Care ("Baycrest") and others
+# (c) 2012-2022, Baycrest Centre for Geriatric Care ("Baycrest") and others
 #
 # This program is free software: you can redistribute it and/or modify it under the
 # terms of the GNU General Public License as published by the Free Software Foundation,
@@ -41,15 +41,15 @@ simulation and the method for running the simulation.
 
 import math
 import time
-
 import numpy
-import scipy.sparse
+
 from tvb.basic.neotraits.api import HasTraits, Attr, NArray, List, Float
 from tvb.basic.profile import TvbProfile
 from tvb.datatypes import cortex, connectivity, patterns
 from tvb.simulator import models, integrators, monitors, coupling
 from tvb.simulator.models.base import Model
-from .backend import BaseBackend, ReferenceBackend
+
+from .backend import ReferenceBackend
 from .common import psutil
 from .history import SparseHistory
 
@@ -197,25 +197,14 @@ class Simulator(HasTraits):
             return True
         return False
 
-    def _configure_integrator_next_step(self):
-        if numpy.all(self.model.state_variables_mask):
-            self.integrate_next_step = self.integrator.integrate
-        else:
+    def configure_integration_for_model(self):
+        self.integrator.configure_boundaries(self.model)
+        if self.model.has_nonint_vars:
             self.integrate_next_step = self.integrator.integrate_with_update
-
-    def _configure_integrator_boundaries(self):
-        if self.model.state_variable_boundaries is not None:
-            indices = []
-            boundaries = []
-            for sv, sv_bounds in self.model.state_variable_boundaries.items():
-                indices.append(self.model.state_variables.index(sv))
-                boundaries.append(sv_bounds)
-            sort_inds = numpy.argsort(indices)
-            self.integrator.bounded_state_variable_indices = numpy.array(indices)[sort_inds]
-            self.integrator.state_variable_boundaries = numpy.array(boundaries).astype("float64")[sort_inds]
+            self.integrator. \
+                reconfigure_boundaries_and_clamping_for_integration_state_variables(self.model)
         else:
-            self.integrator.bounded_state_variable_indices = None
-            self.integrator.state_variable_boundaries = None
+            self.integrate_next_step = self.integrator.integrate
 
     def preconfigure(self):
         """Configure just the basic fields, so that memory can be estimated."""
@@ -225,10 +214,14 @@ class Simulator(HasTraits):
         if self.stimulus:
             self.stimulus.configure()
         self.coupling.configure()
-        self.model.configure()
-        self.integrator.configure()
-        self.integrator.configure_boundaries(self.model)
-        self._configure_integrator_next_step()
+        # ------- Keep this order of configurations ----
+        self.model.configure()  # 1
+        self.integrator.configure()  # 2
+        # Configure integrators' next step computation
+        # and state variables' boundaries and clamping,
+        # based on model attributes  # 3
+        self.configure_integration_for_model()
+        # ----------------------------------------------
         # monitors needs to be a list or tuple, even if there is only one...
         if not isinstance(self.monitors, (list, tuple)):
             self.monitors = [self.monitors]
@@ -244,10 +237,8 @@ class Simulator(HasTraits):
             self.number_of_nodes = self.connectivity.number_of_regions
             self.log.info('Region simulation with %d ROI nodes', self.number_of_nodes)
         else:
-            self._regmap, nc, nsc = self.backend.full_region_map(self.surface, self.connectivity)
-            self.number_of_nodes = nc + nsc
-            self.log.info('Surface simulation with %d vertices + %d non-cortical, %d total nodes',
-                          nc, nsc, self.number_of_nodes)
+            self.number_of_nodes = len(self.surface.region_mapping)
+            self.log.info('Surface simulation with %d total nodes (vertices + non-cortical)', self.number_of_nodes)
 
     def configure(self, full_configure=True):
         """Configure simulator and its components.
@@ -289,31 +280,6 @@ class Simulator(HasTraits):
         # Allow user to chain configure to another call or assignment.
         return self
 
-    def _spatialize_model_parameters(self):
-        # Make sure spatialised model parameters have the right shape (number_of_nodes, 1)
-        # todo: this exclusion list is fragile, consider excluding declarative attrs that are not arrays
-        excluded_params = ("state_variable_range", "state_variable_boundaries", "variables_of_interest",
-                           "noise", "psi_table", "nerf_table", "gid")
-        spatial_reshape = self.model.spatial_param_reshape
-        for param in type(self.model).declarative_attrs:
-            if param in excluded_params:
-                continue
-            region_parameters = getattr(self.model, param)
-            self._map_roi_param_to_surface(param, region_parameters, spatial_reshape)
-            self._reshape_model_param_for_modes(param, spatial_reshape)
-
-    def _reshape_model_param_for_modes(self, param, spatial_reshape):
-        region_parameters = getattr(self.model, param)
-        if region_parameters.size == self.number_of_nodes:
-            new_parameters = region_parameters.reshape(spatial_reshape)
-            setattr(self.model, param, new_parameters)
-
-    def _map_roi_param_to_surface(self, param, region_parameters, spatial_reshape):
-        if self.surface is not None:
-            if region_parameters.size == self.connectivity.number_of_regions:
-                new_parameters = region_parameters[self.surface.region_mapping].reshape(spatial_reshape)
-                setattr(self.model, param, new_parameters)
-
     def _prepare_local_coupling(self):
         if self.surface is None:
             return 0.0
@@ -323,7 +289,7 @@ class Simulator(HasTraits):
         """Compute delayed node coupling values."""
         coupling = self.coupling(step, self.history)
         if self.surface is not None:
-            coupling = coupling[:, self._regmap]
+            coupling = coupling[:, self.surface.region_mapping]
         return coupling
 
     def _prepare_stimulus(self):
@@ -331,7 +297,7 @@ class Simulator(HasTraits):
             stimulus = 0.0
         else:
             # TODO time grid wrong for continuations
-            time = numpy.r_[0.0 : self.simulation_length : self.integrator.dt]
+            time = numpy.r_[0.0: self.simulation_length: self.integrator.dt]
             self.stimulus.configure_time(time.reshape((1, -1)))
             stimulus = numpy.zeros((self.model.nvar, self.number_of_nodes, 1))
             self.log.debug("stimulus shape is: %s", stimulus.shape)
@@ -347,7 +313,7 @@ class Simulator(HasTraits):
     def _loop_update_history(self, step, state):
         """Update history."""
         if self.surface is not None and state.shape[1] > self.connectivity.number_of_regions:
-            state = self.backend.surface_state_to_rois(self._regmap, self.connectivity.number_of_regions, state)
+            state = self.backend.surface_state_to_rois(self.surface.region_mapping, self.connectivity.number_of_regions, state)
         self.history.update(step, state)
 
     def _loop_monitor_output(self, step, state, node_coupling):
@@ -358,16 +324,6 @@ class Simulator(HasTraits):
         if any(outputi is not None for outputi in output):
             return output
 
-    def bound_and_clamp(self, state):
-        # If there is a state boundary...
-        if self.integrator.state_variable_boundaries is not None:
-            # ...use the integrator's bound_state
-            self.integrator.bound_state(state)
-        # If there is a state clamping...
-        if self.integrator.clamped_state_variable_values is not None:
-            # ...use the integrator's clamp_state
-            self.integrator.clamp_state(state)
-
     def __call__(self, simulation_length=None, random_state=None, n_steps=None):
         """
         Return an iterator which steps through simulation time, generating monitor outputs.
@@ -402,9 +358,7 @@ class Simulator(HasTraits):
             if not numpy.issubdtype(type(n_steps), numpy.integer):
                 raise TypeError("Incorrect type for n_steps: %s, expected integer" % type(n_steps))
 
-        for step in range(self.current_step + 1, self.current_step + n_steps + 1):
-            # TODO HO methods compute coupling inside scheme
-            node_coupling = self._loop_compute_node_coupling(step)
+        for step in range(start_step, start_step + n_steps):
             self._loop_update_stimulus(step, stimulus)
             state = self.integrate_next_step(state, self.model, node_coupling, local_coupling, stimulus)
             self._loop_update_history(step, state)
@@ -429,36 +383,43 @@ class Simulator(HasTraits):
         Support 3 possible shapes:
             1) number_of_nodes;
 
-            2) number_of_state_variables; and
+            2) number_of_state_variables or number_of_integrated_state_variables; and
 
-            3) (number_of_state_variables, number_of_nodes).
+            3) (number_of_state_variables or number_of_integrated_state_variables, number_of_nodes).
 
         """
+        # Noise has to have a shape corresponding to only the integrated state variables!
+        good_history_shape = list(self.good_history_shape[1:])
+        good_history_shape[0] = self.model.nintvar
         if self.integrator.noise.ntau > 0.0:
-            self.integrator.noise.configure_coloured(self.integrator.dt,
-                                                     self.good_history_shape[1:])
+            self.integrator.noise.configure_coloured(self.integrator.dt, tuple(good_history_shape))
         else:
-            self.integrator.noise.configure_white(self.integrator.dt,
-                                                  self.good_history_shape[1:])
+            self.integrator.noise.configure_white(self.integrator.dt, tuple(good_history_shape))
 
         if self.surface is not None:
             if self.integrator.noise.nsig.size == self.connectivity.number_of_regions:
                 self.integrator.noise.nsig = self.integrator.noise.nsig[self.surface.region_mapping]
             elif self.integrator.noise.nsig.size == self.model.nvar * self.connectivity.number_of_regions:
+                self.integrator.noise.nsig = \
+                    self.integrator.noise.nsig[self.model.state_variable_mask][:, self.surface.region_mapping]
+            elif self.integrator.noise.nsig.size == self.model.nintvar * self.connectivity.number_of_regions:
                 self.integrator.noise.nsig = self.integrator.noise.nsig[:, self.surface.region_mapping]
 
-        good_nsig_shape = (self.model.nvar, self.number_of_nodes,
-                           self.model.number_of_modes)
+        good_nsig_shape = (self.model.nintvar, self.number_of_nodes, self.model.number_of_modes)
         nsig = self.integrator.noise.nsig
         self.log.debug("Given noise shape is %s", nsig.shape)
         if nsig.shape in (good_nsig_shape, (1,)):
             return
         elif nsig.shape == (self.model.nvar,):
-            nsig = nsig.reshape((self.model.nvar, 1, 1))
+            nsig = nsig[self.model.state_variable_mask].reshape((self.model.nintvar, 1, 1))
+        elif nsig.shape == (self.model.nintvar,):
+            nsig = nsig.reshape((self.model.nintvar, 1, 1))
         elif nsig.shape == (self.number_of_nodes,):
             nsig = nsig.reshape((1, self.number_of_nodes, 1))
         elif nsig.shape == (self.model.nvar, self.number_of_nodes):
-            nsig = nsig.reshape((self.model.nvar, self.number_of_nodes, 1))
+            nsig = nsig[self.model.state_variable_mask].reshape((self.model.nintvar, self.number_of_nodes, 1))
+        elif nsig.shape == (self.model.nintvar, self.number_of_nodes):
+            nsig = nsig.reshape((self.model.nintvar, self.number_of_nodes, 1))
         else:
             msg = "Bad Simulator.integrator.noise.nsig shape: %s"
             self.log.error(msg % str(nsig.shape))
@@ -480,7 +441,7 @@ class Simulator(HasTraits):
         if self.stimulus is not None:
             if self.surface:
                 # NOTE the region mapping of the stimuli should also include the subcortical areas
-                self.stimulus.configure_space(region_mapping=numpy.r_[self.surface.region_mapping, self.connectivity.unmapped_indices(self.surface.region_mapping)])
+                self.stimulus.configure_space(region_mapping=self.surface.region_mapping)
             else:
                 self.stimulus.configure_space()
 
